#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.SolutionCaching {
  [Item("ParallelOperatorProcessor", "An operator that contains multiple operators of which each is applied on the current scope in parallel.")]
  [StorableClass]
  public sealed class ParallelOperatorProcessor : MultiOperator<IOperator> {
    public ValueLookupParameter<BoolValue> ParallelParameter {
      get { return (ValueLookupParameter<BoolValue>)Parameters["Parallel"]; }
    }

    public BoolValue Parallel {
      get { return ParallelParameter.Value; }
      set { ParallelParameter.Value = value; }
    }

    [StorableConstructor]
    private ParallelOperatorProcessor(bool deserializing) : base(deserializing) { }
    private ParallelOperatorProcessor(ParallelOperatorProcessor original, Cloner cloner)
      : base(original, cloner) {
    }
    public ParallelOperatorProcessor()
      : base() {
      Parameters.Add(new ValueLookupParameter<BoolValue>("Parallel", "True if the operators should be applied in parallel on the sub-scopes, otherwise false.", new BoolValue(false)));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParallelOperatorProcessor(this, cloner);
    }

    public override IOperation Apply() {
      OperationCollection next = new OperationCollection(base.Apply());
      OperationCollection inner = new OperationCollection();
      inner.Parallel = Parallel != null && Parallel.Value;
      for (int i = 0; i < Operators.Count(); i++) {
        inner.Add(ExecutionContext.CreateOperation(Operators[i], ExecutionContext.Scope));
      }
      next.Insert(0, inner);
      return next;
    }
  }
}
